import aiohttp
import sys
import asyncio
from . import __version__
from .exceptions import *
from typing import Optional

class HTTP:
    def __init__(self, apikey: str):
        self.BASE_URL = "https://emkc.org/api/v2/piston/"        
        self._headers = {}
        if apikey is not None:
            self._headers["Authorization"] = apikey
        self._headers["User-Agent"] = f"Pyston https://github.com/ffaanngg/pyston {__version__} {sys.version_info} {aiohttp.__version__}"
        self._headers['Content-Type'] = 'application/json'
        self._setup()



    def _setup(self):
        self._loop = asyncio.get_event_loop()
        self._session = aiohttp.ClientSession(headers=self._headers,loop=self._loop)

        
    async def _request(self, method: str, endpoint: str, data: str):
        if data is None:
            async with self._session.request(method,self.BASE_URL + endpoint) as response:
                return response, await response.json()
        else:
            async with self._session.request(method,self.BASE_URL + endpoint, 
                                        data=data) as response:
                return response, await response.json()

    async def get_response(self, method: str, endpoint: str, data: Optional[str] = None):

        response, response_json = await self._request(method, endpoint, data)

        if 300 > response.status >= 200:
            return response_json
        elif response.status == 429:
            raise TooManyRequests("You have been ratelimited.Try again later")
        elif response.status == 500:
            raise InternalServerError("Server failed to respond. Try again later")
        elif response.status == 404:
            raise RuntimeNotFound("Invalid language or version")
        elif response.status == 400:
            raise InvalidArgumentType(response_json["message"])
        else:
            raise UnexpectedError(await response.text())

    async def close(self):
        await self._session.close()

    def __del__(self):
        self._loop.create_task(self.close())



from .exceptions import InvalidLanguage
from typing import Optional,Union
from io import TextIOWrapper
from . import http_handler
import json
from .response import Output

class PystonClient:
    """Pyston client class"""
    def __init__(
        self,
        api_key: Optional[str] = None
    ):

        self._http_session = http_handler.HTTP(api_key)    
    
    async def close_session(self):
        await self._http_session.close()

    async def get_runtimes(self, formatted: Optional[bool] = True):
        """ Returns the runtimes supported by the Piston API"""
        runtimes = await self._http_session.get_response("get", "runtimes/")
        if formatted:
            return json.dumps(runtimes, indent=4)
        else:
            return runtimes

    async def execute(
        self,
        language: str,
        code: Union[str, TextIOWrapper],
        version: Optional[str] = None,
        file_name: Optional[str] = "",
        stdin: Optional[str] = "",
        args: Optional[list] = [],
        compile_timeout: Optional[int] = 10000,
        run_timeout: Optional[int] = 3000,
        compile_memory_limit: Optional[int] = -1,
            run_memory_limit: Optional[int] = -1):
        """Executes the code"""

        payload = {
            "language": language,
            "version": version,
            "stdin": stdin,
            "args": args,
            "compile_timeout": compile_timeout,
            "run_timeout": run_timeout,
            "compile_memory_limit": compile_memory_limit,
            "run_memory_limit": run_memory_limit
        }
        if version is None:
            payload["version"] = await self.get_latest_version(language)
        else:
            payload["version"] = version

        if isinstance(code,TextIOWrapper):
            payload["files"] = [{"name": file_name, "content": code.read()}]
        else:
            payload["files"] = [{"name": file_name, "content": code}]

        output = await self._http_session.get_response("post", 
                                                       "execute/", 
                                                       data=json.dumps(payload))
        return Output(output)

    async def _parse_runtimes(self):
        json_data = await self._http_session.get_response("get", "runtimes/")
        parsed_json = {}
        for element in json_data:
            parsed_json[element["language"]] = element            
        return parsed_json

    async def _parse_aliases(self):
        json_data = await self._http_session.get_response("get", "runtimes/")
        parsed_aliases = {}
        for langauge in json_data:
            version = langauge["version"]
            parsed_aliases[langauge["language"]] = version
            for alias in langauge["aliases"]:
                parsed_aliases[alias] = version
        return parsed_aliases

    async def get_aliases(self, language: str):
        try:
            parsed_runtimes = await self._parse_runtimes()
            return parsed_runtimes[language]["aliases"]
        except KeyError:
            raise InvalidLanguage(f"{language} not found")

    async def get_latest_version(self, language: str):
        try:
            parsed_aliases = await self._parse_aliases()
            return parsed_aliases[language]
        except KeyError:
            raise InvalidLanguage(f"{language} not found")

    async def get_language_info(self, language: str):
        try:
            parsed_runtimes = await self._parse_runtimes()
            return parsed_runtimes[language]
        except KeyError:
            raise InvalidLanguage(f"{language} not found")

    @property
    async def languages(self):
        runtimes = await self._http_session.get_response("get", "runtimes/")
        return [x["language"] for x in runtimes]

    @property
    def endpoints(self):
        return ("runtimes", "execute")
    
    @property
    def base_url(self):
        return "https://emkc.org/api/v2/piston/"




class Output:
    def __init__(self, json_response: dict):
        self._json_response = json_response
        self.langauge = json_response["language"]
        self.version = json_response["language"]
        self.code = json_response["run"]["code"]
        self.signal = json_response["run"]["signal"]
        self.output = json_response["run"]["output"]
        self.stdout = json_response["run"]["stdout"]
        self.stderr = json_response["run"]["stderr"]

    def __repr__(self):
        return self.output

    @property
    def success(self):
        return bool(self.stdout)

    @property
    def raw_json(self):
        return self._json_response



from distutils.core import setup
setup(
  name = 'aiopyston',
  packages = ['pyston'],
  version = '1.0.0',
  license='MIT',
  description = 'An asynchronous API wrapper for the Piston API',
  author = 'Fang',
  url = 'https://github.com/ffaanngg/pyston',
  download_url = 'https://github.com/ffaanngg/pyston/archive/refs/tags/v1.0.0.tar.gz',
  keywords = ['Pyston', 'Piston API', 'API wrapper'],
  install_requires=[
          'aiohttp',
      ],
  classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
  ],
)



class TooManyRequests(Exception):
    pass


class InternalServerError(Exception):
    pass


class UnexpectedError(Exception):
    pass


class RuntimeNotFound(Exception):
    pass


class InvalidLanguage(Exception):
    pass


class InvalidArgumentType(Exception):
    pass
